@@ -12,18 +12,10 @@
 //============================================================================
 // Load Visual Python
 //============================================================================
-(
-    require.specified('base/js/namespace')
-        ? define
-        : function (deps, callback) {
-            'use strict';
-            // if here, the Jupyter namespace hasn't been specified to be loaded.
-            // This means that we're probably embedded in a page,
-            // so we need to make our definition with a specific module name
-            return define('vp_base/js/loadVisualpython', deps, callback);
-        }
-)([
-    'css!vp_base/css/root.css',
+// CHROME: removed code
+define([
+    // CHROME: removed .css extension type
+    __VP_CSS_LOADER__('vp_base/css/root'), // INTEGRATION: unified version of css loader
     'vp_base/js/com/com_Const',
     'vp_base/js/com/com_util',
     'vp_base/js/com/com_Config',
@@ -37,7 +29,7 @@
     //========================================================================
     // Define variable
     //========================================================================
-    var Jupyter;
+    var Jupyter = null;
     var events;
     var liveNotebook = false;
 
@@ -107,7 +99,9 @@
             cfg = $.extend(true, {}, vpConfig.defaultConfig, cfg);
 
         vpFrame = new MainFrame();
-        vpFrame.loadMainFrame();
+        if (vpConfig.extensionType !== 'lab') {
+            vpFrame.loadMainFrame();
+        }
 
         // TODO: hotkey control -> Implement under InputComponent or Event class
         // input:text - hotkey control
@@ -142,8 +136,41 @@
         $(document).on('blur', com_util.wrapSelector('.vp-popup-frame textarea'), function() {
             com_interface.enableOtherShortcut();
         });
+
+        
+        // CHROME: TODO: 2: background <-> vp
+        //======================================================================
+        // Event listener
+        //======================================================================
+        if (window.colab) {
+            document.removeEventListener('vpcomm', _vpcommHandler);
+            document.addEventListener('vpcomm', _vpcommHandler);
+        }
     };
 
+    var _vpcommHandler = function(e) {
+        let detailObj = e.detail;
+        switch (detailObj.type) {
+            // case 'sendBase':
+            //     // get base url of its extension
+            //     vpBase = detailObj.data;
+            //     // initialize vp environment
+            //     vp_init();
+            //     break;
+            case 'toggle':
+                vpLog.display(VP_LOG_TYPE.DEVELOP, 'received from inject - ', e.detail.type, e);
+                // toggle vp_wrapper
+                if (window.vpBase != '' && window.$ && $('#vp_wrapper').length > 0) {
+                    vpFrame.toggleVp();
+                } else {
+                    vpLog.display(VP_LOG_TYPE.DEVELOP, 'No vp...');
+                }
+                break;
+            default:
+                break;
+        }
+    }
+
     var _setGlobalVariables = function() {
         /**
          * visualpython log util
@@ -159,44 +186,24 @@
         /**
          * visualpython config util
          */
-        window.vpConfig = new com_Config();
+        if (window.colab) {
+            // CHROME: added extType as 'colab' // CHROME: FIXME: use window.vpExtType = 'colab'
+            window.vpConfig = new com_Config('colab');
+        } else if (window.vpExtType === 'lab') {
+            // LAB: added extType as 'lab'
+            window.vpConfig = new com_Config('lab');
+        } else {
+            window.vpConfig = new com_Config();
+        }
         window.VP_MODE_TYPE = com_Config.MODE_TYPE;
         /**
          * visualpython kernel
          */
-        window.vpKernel = new com_Kernel();
-    }
-
-    var _checkVersion = function() {
-        // check version timestamp
-        let nowDate = new Date();
-        vpConfig.getData('version_timestamp', 'vpcfg').then(function(data) {
-            let doCheckVersion = false;
-            if (data == undefined) {
-                // no timestamp, check version
-                doCheckVersion = true;
-            } else if (data != '') {
-                let lastCheck = new Date(parseInt(data));
-                let diffCheck_now = new Date(nowDate.getFullYear(), nowDate.getMonth() + 1, nowDate.getDate());
-                let diffCheck_last = new Date(lastCheck.getFullYear(), lastCheck.getMonth() + 1, lastCheck.getDate());
-
-                let diff = Math.abs(diffCheck_now.getTime() - diffCheck_last.getTime());
-                diff = Math.ceil(diff / (1000 * 3600 * 24));
-
-                if (diff >= 1) {
-                    // if More than 1 day passed, check version
-                    doCheckVersion = true;
-                }
-            }
-
-            // check version and update version_timestamp
-            if (doCheckVersion == true) {
-                vpConfig.checkVpVersion(true);
-            }
-
-        }).catch(function(err) {
-            vpLog.display(VP_LOG_TYPE.ERROR, err);
-        })
+        if (vpConfig.extensionType === 'lab') {
+            window.vpKernel = new com_Kernel(vpLab);
+        } else {
+            window.vpKernel = new com_Kernel();
+        }
     }
 
     //========================================================================
@@ -246,20 +253,57 @@
         let cfg = readConfig();
 
         vpConfig.readKernelFunction();
-        _addToolBarVpButton();
+        // CHROME: edited
+        if (Jupyter) {
+            _addToolBarVpButton();
+        }
         _loadVpResource(cfg);
-        _checkVersion();
+        vpConfig.checkVersionTimestamp();
 
-        if (cfg.vp_section_display && vpFrame) {
+        if ((cfg.vp_section_display && vpFrame) 
+            || vpConfig.extensionType === 'colab') { // CHROME: default to display vp
             vpFrame.openVp();
         }
 
         // Operations on kernel restart
-        events.on('kernel_ready.Kernel', function (evt, info) {
-            vpLog.display(VP_LOG_TYPE.LOG, 'vp operations for kernel ready...');
-            // read vp functions
-            vpConfig.readKernelFunction();
-        });
+        if (vpConfig.extensionType === 'notebook') {
+            events.on('kernel_ready.Kernel', function (evt, info) {
+                vpLog.display(VP_LOG_TYPE.LOG, 'vp operations for kernel ready...');
+                // read vp functions
+                vpConfig.readKernelFunction();
+            });
+        } else if (vpConfig.extensionType === 'colab') {
+            // CHROME: ready for colab kernel connected, and restart vp
+            colab.global.notebook.kernel.listen('connected', function(x) { 
+                vpLog.display(VP_LOG_TYPE.LOG, 'vp operations for kernel ready...');
+                // read vp functions
+                vpConfig.readKernelFunction();
+            });
+        } else if (vpConfig.extensionType === 'lab') {
+            // LAB: if widget is ready or changed, ready for lab kernel connected, and restart vp
+            vpLab.shell._currentChanged.connect(function(s1, value) {
+                var { newValue } = value;
+                // kernel restart for notebook and console
+                if (newValue && newValue.sessionContext) {
+                    if (newValue.sessionContext.isReady) {
+                        vpLog.display(VP_LOG_TYPE.LOG, 'vp operations for kernel ready...');
+                        vpConfig.readKernelFunction();
+                        vpConfig.checkVersionTimestamp();
+                    }
+                    newValue.sessionContext._connectionStatusChanged.connect(function(s2, status) {
+                        if (status === 'connected') {
+                            vpLog.display(VP_LOG_TYPE.LOG, 'vp operations for kernel ready...');
+                            vpConfig.readKernelFunction();
+                            vpConfig.checkVersionTimestamp();
+                        }
+                    });
+                } else {
+                    vpLog.display(VP_LOG_TYPE.LOG, 'No widget detected...');
+                }
+            });
+        }
+
+        return vpFrame;
     }
 
     return { initVisualpython: initVisualpython, readConfig: readConfig };
